#
# hosttbl.awk - parse AXFR records from DIG, look at only the A and CNAME
#             records and print a host table format file to stdout.
#

BEGIN {
	soa = "";
	killIPs["127.0.0.1"] = 1;
	A[" "] = " ";			# must guarantee that A is an array
}


#
# If an input line is all-comment, skip it.
#

$1~/^;/ {
	next;
}


# "A" records give us the IP address and hostname info for this domain.
#
# Algorithm:
#  If the IP address is listed in the killIPs array, skip it.
#  A[IPaddr] = host.domain.
#  f[1] = just the hostname with the domain part removed.
#  If host has not already been seen, set B[host.domain.] = host
#  If we've never seen this host.domain. before then set B[host.domain.] = host
#  Else if host is not already in the B[host.domain.] array, then
#	Add host to front of B[host.domain.] array.

NF >= 4 && $3~/^[Aa]$/ {
	if (killIPs[$4] != 1) {
		A[$4] = $1;
		num = split ($1,f,".");
		if (seen[$1] != 1) {
			B[$1] = f[1];
			seen[$1] = 1;
		} else {
			split (B[$1], fB, " ");
			isthere = 0;
			for (i in fB)
				if (fB[i] == f[1])
					isthere = 1;
			if (!isthere)
				B[$1] = f[1] " " B[$1];
		}
	}
	next;
}


# "CNAME" records are hostname aliases for some machine,
# so add this name to the list of names for that host.

NF >= 4 && $3~/^[Cc][Nn][Aa][Mm][Ee]$/ {
	num = split ($1,f,".");
	if (seen[$4] != 1) {
		B[$4] = f[1];
		seen[$4] = 1;
	} else {
		B[$4] = B[$4] " " f[1];
	}
	next;
}


#
# "SOA" record gives us the domain name we are looking in.
# We need the domain name to later eliminate A records for machines
# not in this domain (i.e., ones printed right below NS records).
# Complain if we see more than one SOA record.
#

NF >= 3 && $3~/^[Ss][Oo][Aa]$/ {
	if (soa != "") {
		print "hosttbl.awk: More than one SOA record seen!" | "cat 1>&2";
		exit;
	}
	soa = $1;
	next;
}

#
# "NS" records gives us the hostnames of all nameservers for this domain.
# (we remember these because all of them sneak in their "A" records even
# if they don't belong in this domain!)
#

NF >= 4 && $3~/^[Nn][Ss]$/ {
	ns[$4] = "1";
	next;
}


#
# END - after all input records have been parsed.
# Print all IP addresses and hosts except those that are name-servers
# which are not actually in this domain.  (some nameservers may be
# in this domain, in which case we must print them!)
#

END {

# If we never saw an SOA record, error and exit.

	if (soa == "") {
		print "hosttbl.awk: error, no SOA record seen!" | "cat 1>&2";
		exit;
	}

# For any hostname that is a name-server for this domain,
# if it's not really in this domain then put a "0" in the "ns" array.

	numsoa = split (soa, fsoa, ".");
	realnumsoa = numsoa;
	for (i in ns) {
		good = "1";
		numsoa = realnumsoa;
		numns = split (i, fns, ".");
		while (numns > 0 && numsoa > 0 && good == "1") {
			if (fns[numns] != fsoa[numsoa])
				good = "0";
			numns--;
			numsoa--;
		}
		ns[i] = good;
	}

# Print all IP/host combinations if hostname isn't null.
# Strip off the ending dot on the second-field hostnames.

	for (i in A) {
		if (i == " ")			# throw away placeholder
			continue;

		if (ns[A[i]] != "0") {
			printf ("%s\t", i);
			if (length(i) < 8)	printf ("\t");

			num = split (A[i], fi, ".");
			ip = fi[1];
			for (j=2; j<=num; j++)
				if (fi[j] != "")
					ip = ip "." fi[j];

			printf ("%s\t", ip);
			if (length(ip) < 24)	printf ("\t");
			if (length(ip) < 16)	printf ("\t");
			if (length(ip) < 8)	printf ("\t");

			printf ("%s\n", B[A[i]]);
		}
	}
}
